@@ -244,6 +244,15 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
         }
     }
 
+    async fn ensure_not_completed_state(&mut self) -> Result<(), Error> {
+        match self.state {
+            State::Completed => Err(new_invalid_state_error(
+                "should be in any state other than Completed",
+            )),
+            _ => Ok(()),
+        }
+    }
+
     async fn update_invoice<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
         Ok(self.invoice = Some(ctx.generate_invoice(self).await?))
     }
@@ -351,6 +360,8 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
         ctx: &mut C,
         contact: Contact,
     ) -> Result<(), Error> {
+        self.ensure_not_completed_state().await?;
+
         self.contact = Some(contact);
 
         ctx.update_checkout(self).await
@@ -361,6 +372,8 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
         ctx: &mut C,
         address: Address,
     ) -> Result<(), Error> {
+        self.ensure_not_completed_state().await?;
+
         self.reset_shipping_options();
         self.shipping_address = Some(address);
         if let State::ItemsConfirmed(_) = self.state {
